/*
 * Copyright (c) 2017 - 2018 Dries007. All rights reserved
 */

package net.dries007.mclink;

import com.mojang.authlib.GameProfile;
import net.dries007.mclink.api.APIException;
import net.dries007.mclink.api.Constants;
import net.dries007.mclink.binding.FormatCode;
import net.dries007.mclink.binding.IConfig;
import net.dries007.mclink.binding.IPlayer;
import net.dries007.mclink.common.Log4jLogger;
import net.dries007.mclink.common.MCLinkCommon;
import net.dries007.mclink.common.Player;
import net.dries007.mclink.common.ThreadStartConsumer;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.network.NetHandlerPlayServer;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.management.PlayerList;
import net.minecraft.util.text.*;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.event.FMLServerStartingEvent;
import net.minecraftforge.fml.common.event.FMLServerStoppedEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent.PlayerLoggedInEvent;
import net.minecraftforge.fml.common.network.FMLNetworkEvent;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.util.UUID;

import net.dries007.mclink.common.MCLinkCommon.Side;

/**
 * @author Dries007
 */
@SuppressWarnings("Duplicates")
@Mod(modid = Constants.MODID, name = Constants.MODNAME, useMetadata = true, acceptableRemoteVersions = "*", dependencies = "before:*")
public class MCLink extends MCLinkCommon
{
    private MinecraftServer server;

    @Mod.EventHandler
    public void preInit(final FMLPreInitializationEvent event) throws IConfig.ConfigException, IOException, APIException
    {
        super.setModVersion(event.getModMetadata().version);
        super.setMcVersion(MinecraftForge.MC_VERSION);
        super.setBranding(FMLCommonHandler.instance().getModName());
        super.setLogger(new Log4jLogger(event.getModLog()));
        super.setConfig(new ForgeConfig(event.getSuggestedConfigurationFile()));
        super.setSide(event.getSide().isClient() ? Side.CLIENT : Side.SERVER);
        super.init();
        if (event.getSide().isServer()) MinecraftForge.EVENT_BUS.register(this);
    }

    @Mod.EventHandler
    public void serverStarting(FMLServerStartingEvent event)
    {
        server = event.getServer();
        super.registerCommands(e -> event.registerServerCommand(new CommandWrapper(this, e)));
    }

    @Mod.EventHandler
    public void serverStopped(FMLServerStoppedEvent event)
    {
        server = null;
        super.deInit();
    }

    private Player getPlayerFromEntity(EntityPlayer player)
    {
        return new Player(new SenderWrapper(player), player.getDisplayNameString(), player.getPersistentID());
    }

    @SubscribeEvent
    public void connectEvent(FMLNetworkEvent.ServerConnectionFromClientEvent event)
    {
        if (event.isLocal()) return;
        EntityPlayerMP p = ((NetHandlerPlayServer) event.getHandler()).field_147369_b;
        GameProfile gp = p.func_146103_bH();
        PlayerList pl = server.func_184103_al();
        boolean op = pl.func_152603_m().func_183026_b(gp) || pl.func_152603_m().func_187452_a(gp) > 0;
        boolean wl = pl.func_152599_k().func_152705_a(gp);
        super.checkAuthStatusAsync(getPlayerFromEntity(p), op, wl, new ThreadStartConsumer("checker-" + gp.getId()));
    }

    @SubscribeEvent
    public void loginEvent(PlayerLoggedInEvent event)
    {
        super.login(getPlayerFromEntity(event.player), event.player.func_70003_b(3, Constants.MODID));
    }

    @Override
    protected void kickAsync(IPlayer player, String msg)
    {
        server.func_152344_a(() -> {
            EntityPlayerMP p = server.func_184103_al().func_177451_a(player.getUuid());
            //noinspection ConstantConditions
            if (p != null)
                p.field_71135_a.func_194028_b(new TextComponentString(msg)); // The player may have disconnected before this could happen.
        });
    }

    @Nullable
    @Override
    protected String nameFromUUID(UUID uuid)
    {
        GameProfile gp = server.func_152358_ax().func_152652_a(uuid);
        //noinspection ConstantConditions
        if (gp == null) return null;
        return gp.getName();
    }

    @Override
    public void sendMessage(String message)
    {
        ITextComponent m = new TextComponentTranslation("chat.type.admin", Constants.MODNAME, message);
        m.func_150255_a(new Style().func_150238_a(TextFormatting.GRAY).func_150217_b(true));
        if (server.func_183002_r())
        {
            //noinspection unchecked
            for (EntityPlayer p : server.func_184103_al().func_181057_v())
            {
                if (server.func_184103_al().func_152596_g(p.func_146103_bH())) p.func_145747_a(m);
            }
        }
        server.func_145747_a(m);
    }

    @Override
    public void sendMessage(String message, FormatCode formatCode)
    {
        sendMessage(message);
    }
}
